:- use_package([assertions]).

%% Ciao-specific
:- use_module(library(write)).
:- use_module(library(format)).
:- use_module(library(ttyout)).
:- use_module(library(lists)).

%----------------------------------------------------------------------
% Pretty print papers in .bib format
%----------------------------------------------------------------------

pp_papers_bib(Papers) :-
 	format(user_error,"[Writing out papers",[]),ttyflush,
	do_pp_papers_bib(Papers,0).

do_pp_papers_bib([],Count) :-
 	format(user_error," done. ~w entries processed.]~n",[Count]),ttyflush.
do_pp_papers_bib([H|T],Count) :- 
	pp_paper_bib(H), 
	!, 
 	format(user_error,".",[]),ttyflush,
	NCount is Count + 1,
	do_pp_papers_bib(T,NCount).
do_pp_papers_bib([H|T],Count) :- 
	format(user_error,"[Error: could not format ~w.]~n",[H]),
	ttyflush,
	do_pp_papers_bib(T,Count).

pp_paper_bib(prolog_paper(Keyword,List)) :-
	nonvar(List), %% Otherwise member behaves like repeat...
	member([bibtextype,Type],List),
	!,
	format('~n@~s{~w, ',[Type,Keyword]),
	pp_att_val_pairs_bib(List),
	format('~n}~n',[]),
	flush_output.

% Ignore bibtextype (has already been printed)
pp_att_val_pairs_bib([[bibtextype,_]|T]) :-
	!,
	pp_att_val_pairs_bib(T).
% Format single keywords
pp_att_val_pairs_bib([[Single]|T]) :-
	!,
	pp_att_val_pairs_doit_bib(Single,"yes"),
	format(',',[]),
	pp_att_val_pairs_bib(T).
% ... or ignore single keywords
%% pp_att_val_pairs_bib([[_Single]|T]) :-
%% 	!,
%% 	pp_att_val_pairs_bib(T).
% Was "cliptype" or "pltype" in previous versions
pp_att_val_pairs_bib([[butype,Value]|T]) :-
	!,
	pp_att_val_pairs_doit_bib(butype,Value),
	format(',',[]),
	pp_att_val_pairs_bib(T).
% ... or ingnore it...
%% pp_att_val_pairs_bib([[butype,_]|T]) :-
%% 	!,
%% 	pp_att_val_pairs_bib(T).
% Handle pages specially (this assumes that beginpage appears
% after before endpage -- should be fixed)
pp_att_val_pairs_bib([[beginpage,Bpage]|T]) :-
	!,
	member([endpage,Epage],T),
	append(Bpage,"--",T1),
	append(T1,Epage,Pages),
	pp_att_val_pairs_doit_bib(pages,Pages),
	format(',',[]),
	pp_att_val_pairs_bib(T).
pp_att_val_pairs_bib([[endpage,_]|T]) :-
	!,
	pp_att_val_pairs_bib(T).
% Print all the rest...
pp_att_val_pairs_bib([[Att,Value],[]]) :-
	!,
	pp_att_val_pairs_doit_bib(Att,Value).
pp_att_val_pairs_bib([[]]) :-
	!.
pp_att_val_pairs_bib([[Att,Value]|T]) :-
	T \== [[]],
	pp_att_val_pairs_doit_bib(Att,Value),
	format(',',[]),
	pp_att_val_pairs_bib(T).

pp_att_val_pairs_doit_bib(Att,Value) :-
	format('~n',[]),
	spaces(3),
	format('~q ',[Att]),
	fill_spaces(Att,9),
	write('='),
	pp_val_bib(Att,Value).

fill_spaces(Att,N) :-
	name(Att,AttS),
	length(AttS,L),
	S is N-L,
	spaces(S).

spaces(N) :-
	N =< 0.
spaces(N) :- 
	N > 0,
	write(' '),
	N1 is N-1,
	spaces(N1).

pp_val_bib(Att,Value) :- 
	Att \== author,
	Att \== editor,
	!,
	pp_string_bib(Att,Value).
pp_val_bib(_,List) :-
	format(' {',[]),
	pp_val_authors_bib(List).

pp_val_authors_bib([H]):-
	!,
	pp_string_bib(author,H),
	write('}').
pp_val_authors_bib([H|T]) :-
	pp_string_bib(author,H),
	write(' and '),
	pp_val_authors_bib(T).	

pp_string_bib(author,Value) :-
	!,
	format('~s',[Value]).
%% Comment out this clause to avoid adding automatic capital bracing in titles
%% pp_string_bib(title,Value) :-
%% 	!,
%% 	add_capital_bracing(Value,NValue),
%% 	format(' {~s}',[NValue]).
% Default
pp_string_bib(_,Value) :-
	format(' {~s}',[Value]).

%----------------------------------------------------------------------
:- comment(version_maintenance,dir(version)).

:- comment(version(0*4+23,1999/03/16,14:23*37+'MET'), "Changed
   syntax/1 to use_package/1 for 0.9.  (Manuel Hermenegildo)").

:- comment(version(0*4+1,1998/10/17,20:27*24+'MET DST'), "Fixed bug in
   .bib generation when there were syntax errors in source file.
   (Manuel Hermenegildo)").
%----------------------------------------------------------------------
